#************************************************************************************************
#
#   converter for microcontroller flash internal file system
#
#   This program packs all files located in the directory "files" into one byte array.
#   The byte array is then stored as hexadecimal c-code.
#   This byte array can be included into the internal flash of microcotrollers. There you can
#   access the contents of the file system with dedicated driver functions like fopen, fgetc .. . 
#
#
#************************************************************************************************
#
#    (c) ch 2012
#
#    v0.01   24.4.2012    ch     first version, fixed number of dir entries
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#************************************************************************************************

import os
import glob
from struct import unpack

inputFilePath='./files'
outputFileName='file.c'


#******************************************************************
def toBytes(value):
    return unpack('%sB' % len(value), value)

def toByte(value):
    x=unpack('%sB' % len(value), value)
    if len(x)>0:    
        return x[0]
    else: 
        return 0

#******************************************************************    
class dirEntry:

    name=' '
    startPointer=0
    endPointer=0   

    def setStartPointer(self,sp):
        startPointer=sp
    def setEndPointer(self,ep):
        endPointer=ep  
    def show(self):
        print 'name: ',
        print self.name,
        print '\t size: ',
        print self.endPointer-self.startPointer,
        print ' startpointer: ',
        print self.startPointer,
        print ' endpointer: ',
        print self.endPointer

    def commentFile(self,fc):
        fc.write ('\nname: ')
        fc.write ( self.name)
        fc.write ( '\t size: ')
        fc.write ( str(self.endPointer-self.startPointer ))
        fc.write ( ' startpointer: ')
        fc.write ( str(self.startPointer))
        fc.write ( ' endpointer: ')
        fc.write ( str(self.endPointer))

#******************************************************************
MAXLENGTH=16

def printFileNameAsHex(name):
    for i in range(MAXLENGTH):
        if i<len(name):
            print hex(ord(name[i]))+',',
        else:
            print hex(0)+',',
#******************************************************************
def writeFileNameAsHex(f,name):
    for i in range(MAXLENGTH):
        if i==MAXLENGTH-1:
            f.write(hex(0)+',',)    
        elif i<len(name):
            f.write(hex(ord(name[i]))+',',)
        else:
            f.write(hex(0)+',',)
        
#******************************************************************
def printValueAsHex(v):
    #trick to make type conversion from long to int    
    s=str(v);
    value=int(s)
    print hex(value&0xFF)+',',
    print hex((value>>8)&0xFF)+',',
    print hex((value>>16)&0xFF)+',',
    print hex((value>>24)&0xFF)+',',
#******************************************************************
def writeValueAsHex(f,v):
    #trick to make type conversion from long to int    
    s=str(v);
    value=int(s)
    f.write( hex(value&0xFF)+',')
    f.write( hex((value>>8)&0xFF)+',')
    f.write( hex((value>>16)&0xFF)+',')
    f.write( hex((value>>24)&0xFF)+',')
#******************************************************************
def writeZeros(f,count):
    for i in range(count):
        f.write( hex(0)+',')
#******************************************************************
def writeFileAsHex(filename,outf):
    with open(filename, "rb") as inf:
        byte =inf.read(1)
        outf.write(hex(toByte(byte))+',')
        while byte != "":
            # Do stuff with byte.
            byte = inf.read(1)
            #print hex(toByte(byte))            
            outf.write(hex(toByte(byte))+',')


#******************************************************************
def printDirEntryAsHex(dirEntry):
    d=dirEntry()
    printFileNameAsHex(d.name)


# create directory
directory=[dirEntry()]

DIRENTRYSIZE=32
filePointer=DIRENTRYSIZE
# set filenames
for filename in glob.glob( os.path.join(inputFilePath, '*.*') ):
    d=dirEntry()
    (a,b)=os.path.split(filename)
    d.name=b
    directory.append(d) 
    filePointer=filePointer+DIRENTRYSIZE  

del directory[0]

f = open(outputFileName, 'w+')
f.write('const uint8_t roDrive_content[]={\n')

# set start and end pointers in directory
index=0
for filename in glob.glob( os.path.join(inputFilePath, '*.*') ):
    writeFileNameAsHex(f,directory[index].name)
    f.write('\n')

    directory[index].startPointer=filePointer    
    filePointer=filePointer+os.path.getsize(filename) 
    directory[index].endPointer=filePointer
    filePointer=filePointer+1   
    
    writeValueAsHex(f,directory[index].startPointer)
    writeValueAsHex(f,directory[index].endPointer)
   
    writeZeros(f,8)
    f.write('\n\n')
    index=index+1

#append an empty directory entry to signal directory end
writeZeros(f,DIRENTRYSIZE)
f.write('\n\n')

#write file contents
for filename in glob.glob( os.path.join(inputFilePath, '*.*') ):   
    writeFileAsHex(filename,f)
    f.write('\n')

f.write('\n};\n')
f.write('/*')

#show resutling directory
for dirEn in directory:
    dirEn.commentFile(f)

#show resutling directory
for dirEn in directory:
    dirEn.show()
    
f.write('\n*/;\n')
f.close()

